@@ -1,6 +1,7 @@
 /*   GTimeTracker - a time tracker
  *   Copyright (C) 1997,98 Eckehard Berns
  *   Copyright (C) 2001 Linas Vepstas <linas@linas.org>
+ *   Copyright (C) 2010 Goedson Teixeira Paixao <goedson@debian.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@
 #include <errno.h>
 #include <gconf/gconf.h>
 #include <glade/glade.h>
+#include <gio/gio.h>
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -36,11 +38,9 @@
 #include <sys/wait.h>
 #endif
 
-#include <qof/qof.h>
+#include <qof.h>
 
 #include "app.h"
-#include "ctree.h"
-#include "ctree-gnome2.h"
 #include "cur-proj.h"
 #include "err-throw.h"
 #include "file-io.h"
@@ -53,7 +53,10 @@
 #include "timer.h"
 #include "toolbar.h"
 #include "xml-gtt.h"
+
+#if WITH_DBUS
 #include "dbus.h"
+#endif
 
 char *first_proj_title = NULL;  /* command line over-ride */
 
@@ -156,10 +159,8 @@ unlock_gtt(void)
 #endif
 
 	/* Perform a clean shutdown of QOF subsystem. */
-	qof_query_shutdown ();
-	qof_object_shutdown ();
-	guid_shutdown ();
-	gnc_engine_string_cache_destroy ();
+	qof_close();
+	// qof_log_shutdown();
 
 	/* gnome shutdown */
 #if 0
@@ -212,7 +213,10 @@ post_read_data(void)
 	gtt_post_data_config();
 
 	err_init();
-	ctree_setup(global_ptw, master_list);
+//	ctree_setup(global_ptw, master_list);
+	gtt_projects_tree_populate (projects_tree,
+								gtt_project_list_get_list (master_list),
+								TRUE);
 	gtt_post_ctree_config();
 	menu_set_states();
 	toolbar_set_states();
@@ -225,6 +229,7 @@ post_read_data(void)
 	app_show();
 }
 
+#ifdef THIS_IS_CURRENTLY_UNUSED
 static void
 read_data_err_run_or_abort (GtkDialog *w, gint response_id)
 {
@@ -239,6 +244,7 @@ read_data_err_run_or_abort (GtkDialog *w, gint response_id)
 		gtk_main_quit();
 	}
 }
+#endif /* THIS_IS_CURRENTLY_UNUSED */
 
 static char *
 resolve_old_path (const char * pathfrag)
@@ -294,20 +300,137 @@ resolve_path (const char * pathfrag)
 	return fullpath;
 }
 
-void
-read_data(gboolean reloading) {
+
+static GFile *
+choose_backup_file (char *data_filepath) {
+	GFile *result = NULL;
+
+	GtkWidget * dialog = gtk_file_chooser_dialog_new ("Choose a backup file",
+											NULL,
+											GTK_FILE_CHOOSER_ACTION_OPEN,
+											GTK_STOCK_OPEN,
+											GTK_RESPONSE_ACCEPT,
+											GTK_STOCK_CANCEL,
+											GTK_RESPONSE_REJECT,
+											NULL);
+
+	GFile *data_file = g_file_new_for_path (data_filepath);
+	GFile *data_directory = g_file_get_parent (data_file);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), g_file_get_path (data_directory));
+	g_object_unref (data_file);
+	g_object_unref (data_directory);
+	gint dialog_response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	switch (dialog_response) {
+	case GTK_RESPONSE_ACCEPT:
+		result = gtk_file_chooser_get_file (GTK_FILE_CHOOSER(dialog));
+		break;
+	}
+	gtk_widget_destroy (dialog);
+	return result;
+}
+
+
+static gboolean
+backups_exist (const char *xml_filepath)
+{
+	GFile *data_file = g_file_new_for_path (xml_filepath);
+	GFile *data_dir_file = g_file_get_parent (data_file);
+	char *data_dir_path = g_file_get_path (data_dir_file);
+	char *data_file_name = g_file_get_basename (data_file);
+	GDir *data_dir = g_dir_open (data_dir_path, 0, NULL);
+	gboolean result = FALSE;
+
+	const gchar *file_name = NULL;
+	while ((file_name = g_dir_read_name (data_dir)) != NULL)
+	{
+		if (strcmp(data_file_name, file_name) != 0) {
+			result = TRUE;
+			break;
+		}
+	}
+
+	g_object_unref (data_file);
+	g_object_unref (data_dir_file);
+	g_object_unref (data_dir);
+	g_free (data_dir_path);
+	g_free (data_file_name);
+	return result;
+
+}
+
+
+static gboolean
+try_restoring_backup (char *xml_filepath) {
+
+	GtkWidget *mb;
+	gboolean have_backups = backups_exist (xml_filepath);
+
+	gchar * qmsg = g_strdup_printf("It was not possible to load the data file \"%s\". What do you want to do?", xml_filepath);
+
+	mb = gtk_message_dialog_new (NULL,
+								 GTK_DIALOG_MODAL,
+								 GTK_MESSAGE_ERROR,
+								 GTK_BUTTONS_NONE,
+								 "%s",
+								 qmsg);
+
+	gtk_dialog_add_button (GTK_DIALOG(mb),
+						   _("Create a new file"),
+						   GTK_RESPONSE_YES);
+
+	if (have_backups) {
+		gtk_dialog_add_button (GTK_DIALOG(mb),
+							   _("Load a backup"),
+							   GTK_RESPONSE_NO);
+	}
+
+	gtk_dialog_add_button (GTK_DIALOG(mb),
+						   _("Quit"),
+						   GTK_RESPONSE_CANCEL);
+	
+	gint response = gtk_dialog_run (GTK_DIALOG(mb));
+	gtk_widget_destroy (mb);
+	g_free (qmsg);
+	GFile *backup_file = NULL;
+	GError *error = NULL;
+	gboolean copy_success = FALSE;
+	switch (response) {
+	case GTK_RESPONSE_YES:
+		return FALSE;
+
+	case GTK_RESPONSE_NO:
+		backup_file = choose_backup_file (xml_filepath);
+		if (backup_file != NULL) {
+			copy_success = g_file_copy(backup_file, g_file_new_for_path(xml_filepath), G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &error);
+			if (copy_success) {
+				return TRUE;
+			} else {
+                // TODO Display error message
+				mb = gtk_message_dialog_new (NULL,
+											 GTK_DIALOG_MODAL,
+											 GTK_MESSAGE_ERROR,
+											 GTK_BUTTONS_OK,
+											 _("Could not copy backup file."));
+				gtk_dialog_run (GTK_DIALOG(mb));
+			}
+		} else {
+			exit(1);
+		}
+		return TRUE;
+		break;
+	case GTK_RESPONSE_CANCEL:
+		exit(1);
+		break;
+	}
+	return FALSE;
+}
+
+static gboolean
+read_data_file(char *xml_filepath, GError **error) {
 	GttErrCode xml_errcode;
-	char * xml_filepath;
 	gboolean read_is_ok;
-	char *errmsg, *qmsg;
 
-  if (reloading) {
-      notes_area_set_project(global_na, NULL);
-      gtt_project_list_destroy(master_list);
-      master_list = gtt_project_list_new();
-  }
-
-	xml_filepath = resolve_old_path (config_data_url);
 	/* Try ... */
 	gtt_err_set_code (GTT_NO_ERR);
 	gtt_xml_read_file (xml_filepath);
@@ -344,29 +467,38 @@ read_data(gboolean reloading) {
 	{
 		post_read_data ();
 		g_free (xml_filepath);
-		return;
+		return TRUE;
+	} else {
+		*error = g_error_new (g_quark_from_string ("gtt"), GTT_CANT_OPEN_FILE, "Could not read data file \"%s\"", xml_filepath);
 	}
-
-	/* Else handle an error. */
-	errmsg = gtt_err_to_string (xml_errcode, xml_filepath);
-	qmsg = g_strconcat (errmsg,
-			_("Do you want to continue?"),
-			NULL);
-
-	GtkWidget *mb;
-	mb = gtk_message_dialog_new (NULL,
-	         GTK_DIALOG_MODAL,
-	         GTK_MESSAGE_ERROR,
-	         GTK_BUTTONS_YES_NO,
-	         qmsg);
-	g_signal_connect (G_OBJECT(mb), "response",
-	         G_CALLBACK (read_data_err_run_or_abort),
-	         NULL);
-	gtk_widget_show (mb);
-	g_free (qmsg);
-	g_free (errmsg);
+	return FALSE;
 }
 
+void
+read_data(gboolean reloading) {
+	char * xml_filepath;
+	GError *error = NULL;
+
+  if (reloading) {
+      notes_area_set_project(global_na, NULL);
+      gtt_project_list_destroy(master_list);
+      master_list = gtt_project_list_new();
+  }
+
+	xml_filepath = resolve_old_path (config_data_url);
+
+	while (!read_data_file (xml_filepath, &error)) {
+		if (error != NULL) {
+			if (!try_restoring_backup (xml_filepath)) {
+				break;
+			}
+		}
+	}
+
+	post_read_data ();
+	g_free (xml_filepath);
+	return;
+}
 
 static void
 post_read_config(void)
@@ -416,6 +548,7 @@ read_config(void)
 		         GTK_DIALOG_MODAL,
 		         GTK_MESSAGE_ERROR,
 		         GTK_BUTTONS_YES_NO,
+                 "%s",
 		         qmsg);
 		g_signal_connect (G_OBJECT(mb), "response",
 		         G_CALLBACK (read_config_err_run_or_abort),
@@ -589,6 +722,7 @@ save_properties (void)
 		         GTK_DIALOG_MODAL,
 		         GTK_MESSAGE_WARNING,
 		         GTK_BUTTONS_CLOSE,
+                 "%s",
 		         errmsg);
 		g_signal_connect (G_OBJECT(mb), "response",
 		         G_CALLBACK (gtk_widget_destroy), mb);
@@ -633,6 +767,7 @@ save_projects (void)
 		         GTK_DIALOG_MODAL,
 		         GTK_MESSAGE_WARNING,
 		         GTK_BUTTONS_CLOSE,
+                 "%s",
 		         errmsg);
 		g_signal_connect (G_OBJECT(mb), "response",
 		         G_CALLBACK (gtk_widget_destroy), mb);
@@ -643,8 +778,6 @@ save_projects (void)
 	g_free (xml_filepath);
 }
 
-
-
 /*
  * session management
  */
@@ -655,13 +788,13 @@ save_state(GnomeClient *client, gint phase, GnomeRestartStyle save_style,
 	   gpointer data)
 {
 	char *errmsg;
-	const char *sess_id;
+	// const char *sess_id;
 	char *argv[5];
 	int argc;
 	int x, y, w, h;
 	int rc;
 
-	sess_id  = gnome_client_get_id(client);
+	// sess_id  = gnome_client_get_id(client);
 	if (!app_window) return FALSE;
 
 	gdk_window_get_origin (app_window->window, &x, &y);
@@ -785,14 +918,15 @@ main(int argc, char *argv[])
 	g_signal_connect(G_OBJECT(app_window), "delete_event",
 			   G_CALLBACK(app_quit), NULL);
 
+#if WITH_DBUS
 	gnotime_dbus_setup();
+#endif
 
-	/* Perform QOF-specific initialization. */
-	gnc_engine_get_string_cache();
-	guid_init();
-	qof_object_initialize ();
-	qof_query_init ();
-	qof_book_register ();
+	/* Perform QOF-specific initialization, including dates, objects, query, etc. */
+	qof_init();
+	// Debugging/trace info for the qof query is written to /tmp/qof.trace
+	// qof_log_init();
+	// qof_log_set_level(QOF_MOD_QUERY, QOF_LOG_TRACE);
 	gtt_project_obj_register();
 	master_list = gtt_project_list_new();
 
