@@ -17,13 +17,16 @@
  */
 
 #include "config.h"
+#include <glib-object.h>
 
 #include <glade/glade.h>
 #include <gnome.h>
 
 #include "proj.h"
+#include "props-task.h"
 #include "notes-area.h"
 #include "util.h"
+#include "menus.h"
 
 struct NotesArea_s
 {
@@ -36,13 +39,14 @@ struct NotesArea_s
 	GtkEntry *proj_title;
 	GtkEntry *proj_desc;
 	GtkTextView *proj_notes;
-	
-	GtkEntry *task_memo;
+
+	GtkComboBox *task_combo;
 	GtkTextView *task_notes;
 
 	GtkButton *close_proj;
 	GtkButton *close_task;
 	GtkButton *new_task;
+	GtkButton *edit_task;
 
 	GttProject *proj;
 
@@ -71,7 +75,7 @@ struct NotesArea_s
 	if (na->ignore_events) return;               \
 	                                             \
 	na->ignore_events = TRUE;                    \
-	tsk = gtt_project_get_first_task (na->proj); \
+	tsk = gtt_project_get_current_task (na->proj); \
 	if (NULL == tsk)                             \
 	{                                            \
 		tsk = gtt_task_new();                     \
@@ -86,6 +90,7 @@ struct NotesArea_s
 	}                                            \
 
 
+#ifdef UNUSED_CODE_RIGHT_NOW
 static void
 task_memo_changed (GtkEntry *entry, NotesArea *na)
 {
@@ -94,6 +99,7 @@ task_memo_changed (GtkEntry *entry, NotesArea *na)
 	gtt_task_set_memo (tsk, str);
 	na->ignore_events = FALSE;
 }
+#endif /* UNUSED_CODE_RIGHT_NOW */
 
 /* ============================================================== */
 
@@ -119,7 +125,7 @@ task_notes_changed (GtkTextBuffer *entry, NotesArea *na)
 	}                                     \
 	na->ignore_events = TRUE;             \
 
-	
+
 static void
 proj_title_changed (GtkEntry *entry, NotesArea *na)
 {
@@ -141,6 +147,17 @@ proj_desc_changed (GtkEntry *entry, NotesArea *na)
 	na->ignore_events = FALSE;
 }
 
+/* ============================================================== */
+static GttTask *
+tasks_model_get_task (GtkTreeModel *model, GtkTreeIter *iter)
+{
+
+	GValue v = {G_TYPE_INVALID};
+	gtk_tree_model_get_value (model, iter, 1, &v);
+	GttTask *task = (GttTask *) g_value_get_pointer (&v);
+	g_value_unset (&v);
+	return task;
+}
 
 /* ============================================================== */
 
@@ -156,7 +173,7 @@ proj_notes_changed (GtkTextBuffer *entry, NotesArea *na)
 /* ============================================================== */
 /* This routine will cause pending events to get delivered. */
 
-void 
+void
 gtt_notes_timer_callback (NotesArea *na)
 {
 	if (!na) return;
@@ -187,10 +204,10 @@ close_proj_area (GtkButton *but, NotesArea *na)
 {
 	int hpane_width;
 	int hpane_div;
-	
+
 	hpane_width = GTK_WIDGET(na->hpane)->allocation.width;
 	hpane_div = gtk_paned_get_position (na->hpane);
-	
+
 	if (hpane_div > hpane_width -CLOSED_MARGIN)
 	{
 		int vpane_height;
@@ -208,14 +225,14 @@ close_task_area (GtkButton *but, NotesArea *na)
 {
 	int hpane_width;
 	int hpane_div;
-	
+
 	hpane_width = GTK_WIDGET(na->hpane)->allocation.width;
 	hpane_div = gtk_paned_get_position (na->hpane);
-	
-	/* XXX we really need only the first test, but the second 
-	 * one deals iwth a freaky gtk vpaned bug that makes this 
+
+	/* XXX we really need only the first test, but the second
+	 * one deals iwth a freaky gtk vpaned bug that makes this
 	 * hidden button active.  Whatever.
-	 */ 
+	 */
 	if ((hpane_div < CLOSED_MARGIN) ||
 	    (hpane_div > hpane_width -CLOSED_MARGIN))
 	{
@@ -237,17 +254,63 @@ new_task_cb (GtkButton *but, NotesArea *na)
 	GttTask *tsk;
 	if (NULL == na->proj) return;
 	// if (na->ignore_events) return;
-	
+
 	// na->ignore_events = TRUE;
 	tsk = gtt_task_new();
 	gtt_project_prepend_task (na->proj, tsk);
-	if (NULL != na->task_freeze) gtt_task_thaw (na->task_freeze); 
+	prop_task_dialog_show (tsk);
+	if (NULL != na->task_freeze) gtt_task_thaw (na->task_freeze);
 	gtt_task_freeze (tsk);
 	na->task_freeze = tsk;
 
 	// na->ignore_events = FALSE;
 }
 
+/* ============================================================== */
+static void
+edit_task_cb (GtkButton *but, NotesArea *na)
+{
+	if (NULL == na->proj) return;
+	GttTask *tsk = gtt_project_get_current_task (na->proj);
+	prop_task_dialog_show (tsk);
+	if (NULL != na->task_freeze) gtt_task_thaw (na->task_freeze);
+	gtt_task_freeze (tsk);
+	na->task_freeze = tsk;
+}
+
+/* ============================================================== */
+static void
+task_selected_cb (GtkComboBox *combo, gpointer dialog)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = gtk_combo_box_get_model (combo);
+	NotesArea *na = (NotesArea *) dialog;
+
+	na->ignore_events = TRUE;
+
+	gtk_combo_box_get_active_iter (combo, &iter);
+	GttTask *task = tasks_model_get_task (model, &iter);
+	GttProject *project = gtt_task_get_parent (task);
+
+	gtt_project_set_current_task (project, task);
+
+	const char *str = gtt_task_get_notes (task);
+	if (!str) str = "";
+	xxxgtk_textview_set_text (na->task_notes, str);
+
+	na->ignore_events = FALSE;
+}
+
+/* ============================================================== */
+
+enum {
+	COL_ID,
+	COL_TASK_MEMO,
+	COL_TASK_POINTER,
+	NUM_COLS
+};
+
+
 /* ============================================================== */
 
 #define CONNECT_ENTRY(GLADE_NAME,CB)  ({                           \
@@ -255,7 +318,7 @@ new_task_cb (GtkButton *but, NotesArea *na)
 	entry = GTK_ENTRY(glade_xml_get_widget (gtxml, GLADE_NAME));    \
 	g_signal_connect (G_OBJECT (entry), "changed",                  \
 	                G_CALLBACK (CB), dlg);                          \
-   entry; })
+	entry; })
 
 #define CONNECT_TEXT(GLADE_NAME,CB)  ({                            \
 	GtkTextView *tv;                                                \
@@ -264,7 +327,7 @@ new_task_cb (GtkButton *but, NotesArea *na)
 	buff = gtk_text_view_get_buffer (tv);                           \
 	g_signal_connect (G_OBJECT (buff), "changed",                   \
 	                G_CALLBACK (CB), dlg);                          \
-   tv; })
+	tv; })
 
 
 NotesArea *
@@ -283,24 +346,37 @@ notes_area_new (void)
 	dlg->hpane = GTK_PANED(glade_xml_get_widget (gtxml, "leftright hpane"));
 	dlg->close_proj = GTK_BUTTON(glade_xml_get_widget (gtxml, "close proj button"));
 	dlg->close_task = GTK_BUTTON(glade_xml_get_widget (gtxml, "close diary button"));
-	dlg->new_task = GTK_BUTTON(glade_xml_get_widget (gtxml, "new diary button"));
+	dlg->new_task = GTK_BUTTON(glade_xml_get_widget (gtxml, "new_diary_entry_button"));
+	dlg->edit_task = GTK_BUTTON(glade_xml_get_widget (gtxml, "edit_diary_entry_button"));
 	
 	dlg->proj_title = CONNECT_ENTRY ("proj title entry", proj_title_changed);
 	dlg->proj_desc = CONNECT_ENTRY ("proj desc entry", proj_desc_changed);
-	dlg->task_memo = CONNECT_ENTRY ("diary entry", task_memo_changed);
+	dlg->task_combo = GTK_COMBO_BOX (glade_xml_get_widget (gtxml, "diary_entry_combo"));
+
+	gtk_combo_box_set_model (dlg->task_combo, NULL);
+	GtkCellRenderer *cell;
+	cell = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(dlg->task_combo), cell, TRUE);
+	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT (dlg->task_combo), cell, "text", 0, NULL);
+	g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 
 	dlg->proj_notes = CONNECT_TEXT ("proj notes textview", proj_notes_changed);
 	dlg->task_notes = CONNECT_TEXT ("diary notes textview", task_notes_changed);
-	
 
 	g_signal_connect (G_OBJECT (dlg->close_proj), "clicked",
-	                G_CALLBACK (close_proj_area), dlg);
+					  G_CALLBACK (close_proj_area), dlg);
 
 	g_signal_connect (G_OBJECT (dlg->close_task), "clicked",
-	                G_CALLBACK (close_task_area), dlg);
+					  G_CALLBACK (close_task_area), dlg);
 
 	g_signal_connect (G_OBJECT (dlg->new_task), "clicked",
-	                G_CALLBACK (new_task_cb), dlg);
+					  G_CALLBACK (new_task_cb), dlg);
+
+	g_signal_connect (G_OBJECT (dlg->edit_task), "clicked",
+					  G_CALLBACK (edit_task_cb), dlg);
+
+	g_signal_connect (G_OBJECT (dlg->task_combo), "changed",
+					  G_CALLBACK(task_selected_cb), dlg);
 
 	gtk_widget_show (GTK_WIDGET(dlg->vpane));
 
@@ -312,17 +388,74 @@ notes_area_new (void)
 	return dlg;
 }
 
+
+static void
+notes_area_choose_task (NotesArea *na, GttTask *task)
+{
+
+	GtkTreeIter iter;
+	GtkTreeModel *model = gtk_combo_box_get_model (na->task_combo);
+
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			GttTask *t = tasks_model_get_task (model, &iter);
+
+			if (task == t)
+			{
+				gtk_combo_box_set_active_iter (na->task_combo, &iter);
+				break;
+			}
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+	else
+	{
+		g_warning ("Trying to select task with empty tree model\n");
+	}
+}
+
 /* ============================================================== */
-/* This routine copies data from the data engine, and pushes it 
- * into the GUI.  
- */ 
+void
+combo_model_add_task (gpointer task_p, gpointer model_p)
+{
+	GttTask *task = (GttTask *) task_p;
+	GtkListStore *model = GTK_LIST_STORE (model_p);
+	GtkTreeIter iter;
+
+	gtk_list_store_append (model, &iter);
+
+	gtk_list_store_set (model, &iter,
+						0, gtt_task_get_memo (task),
+						1, task,
+						-1);
+}
+
+/* ============================================================== */
+static GtkTreeModel *
+build_task_combo_model (GttProject *proj)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL (gtk_list_store_new (2,
+															  G_TYPE_STRING,
+															  G_TYPE_POINTER));
+
+	g_list_foreach(gtt_project_get_tasks (proj),
+				   combo_model_add_task,
+				   model);
+	return model;
+}
+
+/* ============================================================== */
+/* This routine copies data from the data engine, and pushes it
+ * into the GUI.
+ */
 
 static void
 notes_area_do_set_project (NotesArea *na, GttProject *proj)
 {
 	const char * str;
 	GttTask *tsk;
-	
+
 	if (!na) return;
 	if (na->ignore_events) return;
 
@@ -331,17 +464,17 @@ notes_area_do_set_project (NotesArea *na, GttProject *proj)
 	 * bug.  So we work around the bug and save cpu time by ignoring
 	 * change events during a mass update. */
 	na->ignore_events = TRUE;
-	
+
 	/* Note Bene its OK to have the proj be null: this has the
 	 * effect of clearing all the fields out.
 	 */
 	na->proj = proj;
-	
+
 	/* Fetch data from the data engine, stuff it into the GUI. */
 	str = gtt_project_get_title (proj);
 	if (!str) str = "";
 	gtk_entry_set_text (na->proj_title, str);
-	
+
 	str = gtt_project_get_desc (proj);
 	if (!str) str = "";
 	gtk_entry_set_text (na->proj_desc, str);
@@ -350,14 +483,15 @@ notes_area_do_set_project (NotesArea *na, GttProject *proj)
 	if (!str) str = "";
 	xxxgtk_textview_set_text (na->proj_notes, str);
 
-	tsk = gtt_project_get_first_task (proj);
-	str = gtt_task_get_memo (tsk);
-	if (!str) str = "";
-	gtk_entry_set_text (na->task_memo, str);
-	
-	str = gtt_task_get_notes (tsk);
-	if (!str) str = "";
-	xxxgtk_textview_set_text (na->task_notes, str);
+	GtkTreeModel *model = build_task_combo_model (proj);
+	gtk_combo_box_set_model (na->task_combo, model);
+	g_object_unref (model);
+
+	tsk = gtt_project_get_current_task (proj);
+	if (tsk == NULL)
+		tsk = gtt_project_get_first_task (proj);
+
+	notes_area_choose_task (na, tsk);
 
 	na->ignore_events = FALSE;
 }
@@ -377,15 +511,15 @@ redraw (GttProject *prj, gpointer data)
 void
 notes_area_set_project (NotesArea *na, GttProject *proj)
 {
-  if (na->proj != NULL) {
-    gtt_project_remove_notifier (na->proj, redraw, na);
-    na->proj = NULL;
-  }
-  if (proj != NULL) {
-    gtt_project_add_notifier (proj, redraw, na);
-  }
+	if (na->proj != NULL) {
+		gtt_project_remove_notifier (na->proj, redraw, na);
+		na->proj = NULL;
+	}
+	if (proj != NULL) {
+		gtt_project_add_notifier (proj, redraw, na);
+	}
 
-  notes_area_do_set_project (na, proj);
+	notes_area_do_set_project (na, proj);
 }
 
 /* ============================================================== */
@@ -411,14 +545,14 @@ projects_tree_selection_changed (GtkTreeSelection *selection, gpointer user_data
 static int
 projects_tree_clicked (GtkWidget *ptree, GdkEvent *event, gpointer data)
 {
-    GdkEventButton *bevent = (GdkEventButton *) event;
-	GttProjectsTree *projects_tree = GTT_PROJECTS_TREE (ptree);
+	GdkEventButton *bevent = (GdkEventButton *) event;
+	// GttProjectsTree *projects_tree = GTT_PROJECTS_TREE (ptree);
 	GtkMenuShell *menu;
 
-    if (!(event->type == GDK_BUTTON_PRESS && bevent->button == 3))
-    {
+	if (!(event->type == GDK_BUTTON_PRESS && bevent->button == 3))
+	{
 		return FALSE;
-    }
+	}
 
 	menu = menus_get_popup ();
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, bevent->time);
@@ -427,7 +561,7 @@ projects_tree_clicked (GtkWidget *ptree, GdkEvent *event, gpointer data)
 
 }
 
-void 
+void
 notes_area_add_projects_tree (NotesArea *nadlg, GttProjectsTree *ptree)
 {
 	if (!nadlg) return;
